//===========================================================================
//
//  Code written for Seeeduino v4.2 runnung at 3.3v
//
//  CRYSTALFONTZ CFAL12832A-022 128x32 OLED in I2C mode
//
//  ref: https://www.crystalfontz.com/product/cfal12832a022y
//       https://www.crystalfontz.com/product/cfal12832a022w
//
//  2017 - 09 - 07 Brent A. Crosby
//===========================================================================
//This is free and unencumbered software released into the public domain.
//
//Anyone is free to copy, modify, publish, use, compile, sell, or
//distribute this software, either in source code form or as a compiled
//binary, for any purpose, commercial or non-commercial, and by any
//means.
//
//In jurisdictions that recognize copyright laws, the author or authors
//of this software dedicate any and all copyright interest in the
//software to the public domain. We make this dedication for the benefit
//of the public at large and to the detriment of our heirs and
//successors. We intend this dedication to be an overt act of
//relinquishment in perpetuity of all present and future rights to this
//software under copyright law.
//
//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
//MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
//IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
//OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
//ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//OTHER DEALINGS IN THE SOFTWARE.
//
//For more information, please refer to <http://unlicense.org/>
//===========================================================================
// C:\Program Files (x86)\Arduino\hardware\arduino\avr\libraries\Wire\src\Wire.h
// C:\Program Files (x86)\Arduino\hardware\arduino\avr\libraries\Wire\src\Wire.cpp
#include <Wire.h>
#include "bitmaps.h"
//============================================================================
/*
The CFAL12832A-022 is a 3.3v device. You need a 3.3v Arduino to operate this
code properly. We used a seeedunio v4.2 set to 3.3v:
http://www.seeedstudio.com/item_detail.html?p_id=2517

Additionally, you will need a 12v bench supply for the OLED power.

  ====================================
  OLED => Connection
  ------------------------------------
   1=NC:       Not Connected
   2=VLSS:     GND
   3=VSS:      GND
   4=NC:       Not Connected
   5=VDD:      3.3v (and 1uF to GND)
   6=BS1:      3.3v (select I2C)
   7=BS2:      GND  (select I2C)
   8=CS#:      GND (I2C: chip always selected)
   9=RES#:     Arduino #9 (purple)
  10=SA0:      GND (Slave address:0x78 write/0x79 read)
                    Arduino style:0x3C write/0x3D read
  11=WR#       GND
  12=RD#:      GND
  13=D0:       SCL Analog 5(add 5K pull-up) white
  14=D1:       SDA(in, tie with D2) Analog 4 green (add 5K pull-up)
  15=D2:       SDA(out, tie with D1) Analog 4 blue
  16=D3:       NC
  17=D4:       NC
  18=D5:       NC
  19=D6:       NC
  20=D7:       NC
  21=IREF:     1.2M ohm to GND (iref=10uA @ VCC=12v)
  22=VCOMH:    1uF to GND
  23=VCC:      12v (bench supply)
  24=NC:       Not Connected
  ====================================
*/
//============================================================================
#define OLED_RESET 9
#define CLR_RESET (digitalWrite(OLED_RESET, LOW))
#define SET_RESET (digitalWrite(OLED_RESET, HIGH))
//============================================================================
// You can create these arrays using Image2Code:
// https://www.crystalfontz.com/product/image2code
// Make the settings as shown in Image2Code_Settings.png
#define NUMBER_OF_SCREENS (7)
const SCREEN_IMAGE *const screens[NUMBER_OF_SCREENS] PROGMEM=
  {
  &Logo_128x32,
  &Dimensions_128x32,
  &Media_Buttons_128x32,
  &Big_Part_Number_128x32,
  &Small_Part_Number_128x32,
  &Text_6x8_128x32,
  &Text_8x8_128x32
  };
//============================================================================
void I2C_sendCommand(uint8_t payload)
  {
  Wire.beginTransmission(0x3C); //send start & Slave address
  Wire.write(0x00);		          //Control Byte - Command
  Wire.write(payload);	        //payload
  Wire.endTransmission();
  }
//============================================================================
void I2C_sendData(uint8_t payload)
  {
  Wire.beginTransmission(0x3C);	//send start & Slave address
  Wire.write(0xC0);		          //Control Byte - Data (non-continued)
  Wire.write(payload);          //payload
  Wire.endTransmission();
  }
//============================================================================
// fef: https://www.crystalfontz.com/controllers/SolomonSystech/SSD1305/
#define SSD1305Z_00_SET_LOWER_COLUMN_ADDRESS_BIT   (0x00)
#define SSD1305Z_10_SET_UPPER_COLUMN_ADDRESS_BIT   (0x10)
#define SSD1305Z_SET_MEMORY_ADDRESS_MODE_PREFIX_20 (0x20)
#define SSD1305Z_SET_COLUMN_ADDRESS_PREFIX_21      (0x21)
#define SSD1305Z_SET_PAGE_ADDRESS_PREFIX_22        (0x22)
#define SSD1305Z_40_DISPLAY_START_LINE_BIT         (0x40)
#define SSD1305Z_CONTRAST_BANK0_PREFIX_81          (0x81)
#define SSD1305Z_BRIGHTNESS_AREA_PREFIX_82         (0x82)
#define SSD1305Z_SET_LUT_PREFIX_91                 (0x91)
#define SSD1305Z_SET_BANK_COLOR_01_16_PREFIX_92    (0x92)
#define SSD1305Z_SET_BANK_COLOR_17_32_PREFIX_93    (0x93)
#define SSD1305Z_SEG0_IS_COL_0_A0                  (0xA0)
#define SSD1305Z_SEG0_IS_COL_127_A1                (0xA1)
#define SSD1305Z_ENTIRE_DISPLAY_NORMAL_A4          (0xA4)
#define SSD1305Z_ENTIRE_DISPLAY_FORCE_ON_A5        (0xA5)
#define SSD1305Z_INVERSION_NORMAL_A6               (0xA6)
#define SSD1305Z_INVERSION_INVERTED_A7             (0xA7)
#define SSD1305Z_MULTIPLEX_RATIO_PREFIX_A8         (0xA8)
#define SSD1305Z_DIM_MODE_PREFIX_AB                (0xAB)
#define SSD1305Z_DISPLAY_ON_DIM_AC                 (0xAC)
#define SSD1305Z_MASTER_CONFIGURATION_PREFIX_AD    (0xAD)
#define SSD1305Z_DISPLAY_OFF_SLEEP_RESET_AE        (0xAE)
#define SSD1305Z_DISPLAY_ON_NORMAL_AF              (0xAF)
#define SSD1305Z_B0_SET_PAGE_START_ADDRESS_BIT     (0xB0)
#define SSD1305Z_SCAN_DIR_UP_C0                    (0xC0)
#define SSD1305Z_SCAN_DIR_DOWN_C8                  (0xC8)
#define SSD1305Z_DISPLAY_OFFSET_PREFIX_D3          (0xD3)
#define SSD1305Z_CLOCK_DIVIDE_PREFIX_D5            (0xD5)
#define SSD1305Z_AREA_COLOR_LOW_POWER_PREFIX_D8    (0xD8)
#define SSD1305Z_PRECHARGE_PERIOD_PREFIX_D9        (0xD9)
#define SSD1305Z_COM_CONFIG_PREFIX_DA              (0xDA)
#define SSD1305Z_VCOMH_DESELECT_PREFIX_DB          (0xDB)
#define SSD1305Z_ENTER_RMW_E0                      (0xE0)
#define SSD1305Z_EXIT_RMW_EE                       (0xEE)
//============================================================================
void Set_Address(uint8_t column, uint8_t page)
  {
  //The CFAL12832A-022 has a 4-pixel horizontal offset.
  column+=4;
  //Set column-lower nibble
  I2C_sendCommand(SSD1305Z_00_SET_LOWER_COLUMN_ADDRESS_BIT|(column&0x0F));
  //Set column-upper nibble
  I2C_sendCommand(SSD1305Z_10_SET_UPPER_COLUMN_ADDRESS_BIT|((column>>4)&0x0F));
  //Set page address, limiting from 0 to 7
  I2C_sendCommand(SSD1305Z_B0_SET_PAGE_START_ADDRESS_BIT|(page&0x0F));
  }
//============================================================================
void show_128_x_32_bitmap(const SCREEN_IMAGE *OLED_image)
  {
  // ~ 16.26 mS normal I2C_sendData() calls are ~45 mS
  uint8_t
    column;
  uint8_t
    horiz_block;
  uint8_t
    byte_count;

  uint8_t
    row;

  for(row=0;row<4;row++)
    {
    Set_Address(0,row);
    //Arduino TWI Library is max 32 byte transfer by default.
    //Break into 8 I2C transfers of 16 bytes each.
    column=0;
    for(horiz_block=0;horiz_block<8;horiz_block++)
      {
      Wire.beginTransmission(0x3C);  //send start & Slave address
      Wire.write(0x40);              //Control Byte - Data (continued)
      for(byte_count=0;byte_count<16;byte_count++)
        {
        //Read this byte from the program memory / flash
        Wire.write(pgm_read_byte( &(OLED_image  ->bitmap_data[row][column]) ));
        column++;
        }
      Wire.endTransmission();
      }
    }
  }
//============================================================================
void Initialize_CFAL12832A(void)
  {
  //Thump the reset.  
  _delay_ms(1);
  CLR_RESET;
  _delay_ms(1);
  SET_RESET;
  _delay_ms(1);

  //Set the display to sleep mode for the rest of the init.
  I2C_sendCommand(SSD1305Z_DISPLAY_OFF_SLEEP_RESET_AE);
  // Master Configuration, Select external VCC supply
  I2C_sendCommand(SSD1305Z_MASTER_CONFIGURATION_PREFIX_AD);  
  I2C_sendCommand(0x8E);
  
  I2C_sendCommand(SSD1305Z_MULTIPLEX_RATIO_PREFIX_A8);
  // Default =>0x1F(1/32 Duty) 0x3F (1/64 Duty)
  I2C_sendCommand(0x1F); 

  I2C_sendCommand(SSD1305Z_DISPLAY_OFFSET_PREFIX_D3);
  //00H Reset
  I2C_sendCommand(0x00);  

  //Set Display Start Line to 0
  I2C_sendCommand(SSD1305Z_40_DISPLAY_START_LINE_BIT | 0x00);

  I2C_sendCommand(SSD1305Z_INVERSION_NORMAL_A6);
  
  I2C_sendCommand(SSD1305Z_VCOMH_DESELECT_PREFIX_DB);
  I2C_sendCommand(0x3C); //~0.83xVCC

  I2C_sendCommand(SSD1305Z_ENTIRE_DISPLAY_NORMAL_A4);

  I2C_sendCommand(SSD1305Z_CONTRAST_BANK0_PREFIX_81);
  I2C_sendCommand(0xBF);

  I2C_sendCommand(SSD1305Z_CLOCK_DIVIDE_PREFIX_D5);
  I2C_sendCommand(0xF0);  //105HZ

  I2C_sendCommand(SSD1305Z_AREA_COLOR_LOW_POWER_PREFIX_D8);
  I2C_sendCommand(0x05); //MONO Mode and Low Power display Mode

  //Set Segment Re-Map Default => 0xA0
  //0xA1 (0x01) => Column Address 0 Mapped to SEG131
  I2C_sendCommand(SSD1305Z_SEG0_IS_COL_127_A1);

  //Set COM Output Scan Direction Default => 0xC0
  //0xC8 (0x08) => Scan from COM63 to 0
  I2C_sendCommand(SSD1305Z_SCAN_DIR_DOWN_C8);
  
  I2C_sendCommand(SSD1305Z_COM_CONFIG_PREFIX_DA); 
  I2C_sendCommand(0x12); //Alternative COM Pin

  I2C_sendCommand(SSD1305Z_PRECHARGE_PERIOD_PREFIX_D9);
  I2C_sendCommand(0xF2); //Refer to SPEC Page34
  I2C_sendCommand(0xFF); //

  I2C_sendCommand(SSD1305Z_DISPLAY_ON_NORMAL_AF); // Display ON
  }
//============================================================================
uint8_t
  gear_dir;
//----------------------------------------------------------------------------
void setup( void )
  {
  // Initialize digital pin 4 as an output for the reset.
  pinMode(OLED_RESET, OUTPUT);

  //Get the I2C going
  Wire.begin();

  //Thanks to Limor / Adafruit for the hint ;-)   
  TWBR = 12; // upgrade to 400KHz!

  //Fire up the I2C OLED
  Initialize_CFAL12832A();

  gear_dir=0;
  }
//----------------------------------------------------------------------------
void  loop(void)
  {
  uint8_t
    current_screen;
  //Put up some bitmaps from flash
  for(current_screen=0;current_screen<NUMBER_OF_SCREENS;current_screen++)
    {
    show_128_x_32_bitmap((SCREEN_IMAGE *)pgm_read_word(&screens[current_screen]));
    //Wait a bit . . .
    _delay_ms(1500);
    if(current_screen < 2)
      {
      //Wait a little extra for the logo & dimensions
      _delay_ms(750);
      }
    }
  }
//============================================================================

